#include "reg_uC89C4xx.h"

#define RS       P3_7
#define RW       P3_6
#define E        P3_5
#define LCD_DATA P1

#define BF P1_7

#define BUFFER 0x08

#define CLR_E   (E  = 0)
#define SETB_E  (E  = 1)
#define SETB_RS (RS = 1)
#define CLR_RS  (RS = 0)
#define SETB_RW (RW = 1)
#define CLR_RW  (RW = 0)
#define SETB_BF (BF = 1)
#define CLR_BF  (BF = 0)

void send_string      (unsigned char *string);
void send_cmd         (unsigned char cmd);
void send_data        (unsigned char data);
void init_LCD         ();
void delay            (void);
void send_chars_map   (unsigned char len, unsigned char *chars_map);
void load_game_chars  (void);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void load_game_chars (void){
    unsigned char smile_lup   [] = {0x00, 0x00, 0x01, 0x03, 0x03, 0x07, 0x07, 0x07};
    unsigned char smile_lupc  [] = {0x07, 0x1F, 0x15, 0x1B, 0x15, 0x1F, 0x1F, 0x1E};/* 0x7, 0x1f, 0x1f, 0x15, 0x1b, 0x15, 0x1f, 0x1e */
    unsigned char smile_rupc  [] = {0x1C, 0x1F, 0x15, 0x1B, 0x15, 0x1F, 0x1F, 0x0F};/* 0x1c, 0x1f, 0x1f, 0x15, 0x1b, 0x15, 0x1f, 0xf */
    unsigned char smile_rup   [] = {0x00, 0x00, 0x10, 0x18, 0x18, 0x1C, 0x1C, 0x1C};
    unsigned char smile_ldw   [] = {0x07, 0x07, 0x07, 0x03, 0x03, 0x01, 0x00, 0x00};
    unsigned char smile_ldwc  [] = {0x1E, 0x1F, 0x17, 0x17, 0x1B, 0x1C, 0x1F, 0x07};
    unsigned char smile_rdwc  [] = {0x0F, 0x1F, 0x1D, 0x1D, 0x1B, 0x07, 0x1F, 0x1C};
    unsigned char smile_rdw   [] = {0x1C, 0x1C, 0x1C, 0x18, 0x18, 0x10, 0x00, 0x00};

    send_cmd (0x40);
    send_chars_map (BUFFER, smile_lup);
    send_chars_map (BUFFER, smile_lupc);
    send_chars_map (BUFFER, smile_rupc);
    send_chars_map (BUFFER, smile_rup);
    send_chars_map (BUFFER, smile_ldw);
    send_chars_map (BUFFER, smile_ldwc);
    send_chars_map (BUFFER, smile_rdwc);
    send_chars_map (BUFFER, smile_rdw);
}

void send_chars_map (unsigned char len, unsigned char *chars_map){
    unsigned char i = 0;

    while (i < len){             /*for each string index*/
        send_data (*chars_map);  /*send solo char for each CGRAM line*/
        chars_map++;             /*move to next index*/
        i++;                     /*move to next iteraction*/
    }
}

void delay (void){
    unsigned int i;
    unsigned int j;
    unsigned int k;
    unsigned int l = 0x00;

    for (i = 0x00; i < 0xFF; i++)
        for (j = 0x00; j < 0x4A; j++)
            for (k = 0x00; k < 0x0A; k++);
                l++;
}

void send_string (unsigned char *string){ /*send a string to the LCD DDRAM*/
    while (*string){
        send_data (*string);
        ++string;
    }
}

void send_cmd (unsigned char cmd){
    LCD_DATA = cmd;
    CLR_RS;
    CLR_RW;
    SETB_E;
    //delay ();
    CLR_E;
}

void send_data (unsigned char data){ /*send some data/char to LCD*/
    LCD_DATA = data;
    SETB_RS;
    CLR_RW;
    SETB_E;
    //delay ();
    CLR_E;
}

void init_LCD (){
    send_cmd (0x38);/*configure LCD with 2 lines, 5x7 matrix*/
    send_cmd (0x0F);/*Display ON, Cursor Blinking, Cursor ON*/
    send_cmd (0x01);/*Clear display*/
    send_cmd (0x06);/*Increment cursor mode*/
}

void main (void) {
    unsigned char smile_ldwcs [] = {0x1E, 0x1F, 0x1D, 0x1B, 0x17, 0x17, 0x1F, 0x07};
    unsigned char smile_rdwcs [] = {0x0F, 0x1F, 0x07, 0x1B, 0x1D, 0x1D, 0x1F, 0x1C};
    unsigned char cgram_clear [] = {0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00};
    init_LCD ();
    IE   = 0x81;
    TCON = 0x01;
    while (0x01) {
        load_game_chars ();
        send_cmd  (0x87);
        send_data (0x00);
        send_data (0x01);
        send_data (0x02);
        send_data (0x03);
        send_cmd  (0xC7);
        send_data (0x04);
        send_data (0x05);
        send_data (0x06);
        send_data (0x07);
        send_cmd  (0x41);
        send_chars_map (BUFFER, cgram_clear);
        send_cmd  (0x46);
        send_chars_map (BUFFER, smile_ldwcs);
        send_cmd  (0x47);
        send_chars_map (BUFFER, cgram_clear);
        send_cmd  (0x47);
        send_chars_map (BUFFER, smile_rdwcs);
        send_cmd  (0xC9);
        send_data (0x06);
        send_cmd  (0xCA);
        send_data (0x07);
    }

}

void ISR_EX0 (void) __interrupt (0x00) {
    send_cmd (0x01); /*Clear LCD command*/
}

_sdcc_external_startup (void) {
    return 0x01;
}